<?php

namespace FiloBlu\ExtElasticsearch\Plugin;

use Magento\Elasticsearch\SearchAdapter\Query\Builder\Match;

/**
 * Class BuilderMatchPlugin
 * @package FiloBlu\ExtElasticsearch\Plugin
 */
class BuilderMatchPlugin
{
    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $_logger;

    /**
     * BuilderMatchPlugin constructor.
     * @param \Psr\Log\LoggerInterface $logger
     */
    public function __construct(\Psr\Log\LoggerInterface $logger)
    {
        $this->_logger = $logger;
    }

    /**
     * Remove filed => '*' from processing
     *
     * @param array $matches
     * @param array $queryValue
     * @param Match $subject
     *
     * @return array|null
     */
    public function beforeBuildQueries(Match $subject, $matches, $queryValue)
    {
        $filteredMatches = [];

        foreach ($matches as $match) {

            if (isset($match['field']) && $match['field'] === '*') {
                continue;
            }

            $filteredMatches[] = $match;
        }

        return [$filteredMatches, $queryValue];
    }

    /**
     * Escape a value for special query characters such as ':', '(', ')', '*', '?', etc.
     *
     * @param Match $subject
     * @param string $value
     * @return string
     */
    protected function aroundEscape(Match $subject, $value)
    {
        $pattern = '/(\+|&&|\|\||!|\(|\)|\{|}|\[|]|\^|"|~|\*|\?|:|\\\)/';
        $replace = '\\\$1';

        return preg_replace($pattern, $replace, $value);
    }
}